# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blockexplorer/blockexplorer.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from vega.commands.v1 import (
    transaction_pb2 as vega_dot_commands_dot_v1_dot_transaction__pb2,
)
from protoc_gen_openapiv2.options import (
    annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2,
)


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!blockexplorer/blockexplorer.proto\x12\x14\x62lockexplorer.api.v1\x1a"vega/commands/v1/transaction.proto\x1a.protoc-gen-openapiv2/options/annotations.proto"+\n\x15GetTransactionRequest\x12\x12\n\x04hash\x18\x01 \x01(\tR\x04hash"]\n\x16GetTransactionResponse\x12\x43\n\x0btransaction\x18\x01 \x01(\x0b\x32!.blockexplorer.api.v1.TransactionR\x0btransaction"\x8e\x02\n\x17ListTransactionsRequest\x12\x14\n\x05limit\x18\x01 \x01(\rR\x05limit\x12\x1b\n\x06\x62\x65\x66ore\x18\x02 \x01(\tH\x00R\x06\x62\x65\x66ore\x88\x01\x01\x12\x19\n\x05\x61\x66ter\x18\x03 \x01(\tH\x01R\x05\x61\x66ter\x88\x01\x01\x12T\n\x07\x66ilters\x18\x04 \x03(\x0b\x32:.blockexplorer.api.v1.ListTransactionsRequest.FiltersEntryR\x07\x66ilters\x1a:\n\x0c\x46iltersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\t\n\x07_beforeB\x08\n\x06_after"a\n\x18ListTransactionsResponse\x12\x45\n\x0ctransactions\x18\x03 \x03(\x0b\x32!.blockexplorer.api.v1.TransactionR\x0ctransactions"\xe2\x01\n\x0bTransaction\x12\x14\n\x05\x62lock\x18\x01 \x01(\x04R\x05\x62lock\x12\x14\n\x05index\x18\x02 \x01(\rR\x05index\x12\x12\n\x04hash\x18\x03 \x01(\tR\x04hash\x12\x1c\n\tsubmitter\x18\x04 \x01(\tR\tsubmitter\x12\x12\n\x04type\x18\x05 \x01(\tR\x04type\x12\x12\n\x04\x63ode\x18\x06 \x01(\rR\x04\x63ode\x12\x16\n\x06\x63ursor\x18\x07 \x01(\tR\x06\x63ursor\x12\x35\n\x07\x63ommand\x18\x08 \x01(\x0b\x32\x1b.vega.commands.v1.InputDataR\x07\x63ommand2\xfa\x01\n\x14\x42lockExplorerService\x12m\n\x0eGetTransaction\x12+.blockexplorer.api.v1.GetTransactionRequest\x1a,.blockexplorer.api.v1.GetTransactionResponse"\x00\x12s\n\x10ListTransactions\x12-.blockexplorer.api.v1.ListTransactionsRequest\x1a..blockexplorer.api.v1.ListTransactionsResponse"\x00\x42xZ5code.vegaprotocol.io/vega/protos/blockexplorer/api/v1\x92\x41>\x12#\n\x18Vega block explorer APIs2\x07v0.59.0\x1a\x13lb.testnet.vega.xyz*\x02\x01\x02\x62\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "blockexplorer.blockexplorer_pb2", globals()
)
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z5code.vegaprotocol.io/vega/protos/blockexplorer/api/v1\222A>\022#\n\030Vega block explorer APIs2\007v0.59.0\032\023lb.testnet.vega.xyz*\002\001\002"
    _LISTTRANSACTIONSREQUEST_FILTERSENTRY._options = None
    _LISTTRANSACTIONSREQUEST_FILTERSENTRY._serialized_options = b"8\001"
    _GETTRANSACTIONREQUEST._serialized_start = 143
    _GETTRANSACTIONREQUEST._serialized_end = 186
    _GETTRANSACTIONRESPONSE._serialized_start = 188
    _GETTRANSACTIONRESPONSE._serialized_end = 281
    _LISTTRANSACTIONSREQUEST._serialized_start = 284
    _LISTTRANSACTIONSREQUEST._serialized_end = 554
    _LISTTRANSACTIONSREQUEST_FILTERSENTRY._serialized_start = 475
    _LISTTRANSACTIONSREQUEST_FILTERSENTRY._serialized_end = 533
    _LISTTRANSACTIONSRESPONSE._serialized_start = 556
    _LISTTRANSACTIONSRESPONSE._serialized_end = 653
    _TRANSACTION._serialized_start = 656
    _TRANSACTION._serialized_end = 882
    _BLOCKEXPLORERSERVICE._serialized_start = 885
    _BLOCKEXPLORERSERVICE._serialized_end = 1135
# @@protoc_insertion_point(module_scope)
