# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vega/data/v1/spec.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17vega/data/v1/spec.proto\x12\x0cvega.data.v1"n\n\x06\x46ilter\x12+\n\x03key\x18\x01 \x01(\x0b\x32\x19.vega.data.v1.PropertyKeyR\x03key\x12\x37\n\nconditions\x18\x02 \x03(\x0b\x32\x17.vega.data.v1.ConditionR\nconditions"\xdf\x01\n\x0bPropertyKey\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x32\n\x04type\x18\x02 \x01(\x0e\x32\x1e.vega.data.v1.PropertyKey.TypeR\x04type"\x87\x01\n\x04Type\x12\x14\n\x10TYPE_UNSPECIFIED\x10\x00\x12\x0e\n\nTYPE_EMPTY\x10\x01\x12\x10\n\x0cTYPE_INTEGER\x10\x02\x12\x0f\n\x0bTYPE_STRING\x10\x03\x12\x10\n\x0cTYPE_BOOLEAN\x10\x04\x12\x10\n\x0cTYPE_DECIMAL\x10\x05\x12\x12\n\x0eTYPE_TIMESTAMP\x10\x06"\x93\x02\n\tCondition\x12<\n\x08operator\x18\x01 \x01(\x0e\x32 .vega.data.v1.Condition.OperatorR\x08operator\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value"\xb1\x01\n\x08Operator\x12\x18\n\x14OPERATOR_UNSPECIFIED\x10\x00\x12\x13\n\x0fOPERATOR_EQUALS\x10\x01\x12\x19\n\x15OPERATOR_GREATER_THAN\x10\x02\x12"\n\x1eOPERATOR_GREATER_THAN_OR_EQUAL\x10\x03\x12\x16\n\x12OPERATOR_LESS_THAN\x10\x04\x12\x1f\n\x1bOPERATOR_LESS_THAN_OR_EQUAL\x10\x05\x42/Z-code.vegaprotocol.io/vega/protos/vega/data/v1b\x06proto3'
)

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "vega.data.v1.spec_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b"Z-code.vegaprotocol.io/vega/protos/vega/data/v1"
    _FILTER._serialized_start = 41
    _FILTER._serialized_end = 151
    _PROPERTYKEY._serialized_start = 154
    _PROPERTYKEY._serialized_end = 377
    _PROPERTYKEY_TYPE._serialized_start = 242
    _PROPERTYKEY_TYPE._serialized_end = 377
    _CONDITION._serialized_start = 380
    _CONDITION._serialized_end = 655
    _CONDITION_OPERATOR._serialized_start = 478
    _CONDITION_OPERATOR._serialized_end = 655
# @@protoc_insertion_point(module_scope)
