"""
This is the Vega gRPC API client.

Code generated by `generate_init.py`. DO NOT EDIT.
"""

from . import (
    api,
    checkpoint,
    commands,
    events,
    data,
    snapshot,
    wallet,
    assets_pb2 as assets,
    assets_pb2_grpc as assets_grpc,
    chain_events_pb2 as chain_events,
    chain_events_pb2_grpc as chain_events_grpc,
    data_source_pb2 as data_source,
    data_source_pb2_grpc as data_source_grpc,
    governance_pb2 as governance,
    governance_pb2_grpc as governance_grpc,
    markets_pb2 as markets,
    markets_pb2_grpc as markets_grpc,
    oracle_pb2 as oracle,
    oracle_pb2_grpc as oracle_grpc,
    vega_pb2 as vega,
    vega_pb2_grpc as vega_grpc,
)

__all__ = [
    "api",
    "checkpoint",
    "commands",
    "events",
    "data",
    "snapshot",
    "tm",
    "wallet",
    "assets",
    "assets_grpc",
    "chain_events",
    "chain_events_grpc",
    "data_source",
    "data_source_grpc",
    "governance",
    "governance_grpc",
    "markets",
    "markets_grpc",
    "oracle",
    "oracle_grpc",
    "vega",
    "vega_grpc",
]
